<?php
/*
 * myNonOverlapingDateValidator is a validator módule to check that periods
 * passed to add a new period to a period table are not overlaped
*/

class myNonOverlapingDateValidator extends sfValidator
{

  public function initialize($context, $parameters = null)
  {
    // initialize parent
    parent::initialize($context, $parameters);
 
    // set defaults
    $this->setParameter('overlaping_error', 'Invalid date');
 
    $this->getParameterHolder()->add($parameters);
 
    return true;
  }
  
  /*
   * Param needed (+ are mandatory, * one of this is mandatory):
   * +class_name: the name for the class where we look for periods
   * +start_date_db: the start date of the period in the db table
   * +end_date_db: the end date of the period in the db table
   * +start_date_field: the start date form field
   * +end_date_field: the end date form field
   * +start_date_error_message: the returned error message
   * +end_date_error_message: the returned error message
   * +inside_error_message: the returned error message
   * -exception_id: if there is an exception row in the table (Example: in 
   *  edition the row edited), it takes the field of the form which contains it) 
   * -criteria: some additional criteria for propel selection. This aditional 
   *  criteria must be noted this way:
   * criteria:
   *  criteria1:
   *    +column: column name for criteria checking
   *    *value: value for criteria
   *    *field: field of the form where the value is defined
   *    -criteria: criteria pattern (see symfony or propel documentation) 
   */
   
  public function execute(&$value, &$error)
  { 
  	/*
  	 * Recover data from form and validation file
  	 */
  	//Recover culture for date transforming
    $culture = $this->getContext()->getUser()->getCulture();
    
    //Recover table name and checked columns
    $className = $this->getParameter('class_name');
    $startDateDb = $this->getParameter('start_date_db');
    $endDateDb = $this->getParameter('end_date_db');
    
    //recover data from the form
    $startDateField = $this->getParameter('start_date_field');
    $endDateField = $this->getParameter('end_date_field');
    $startDate=$this->getContext()->getRequest()->getParameter($startDateField);
    $endDate=$this->getContext()->getRequest()->getParameter($endDateField);
    if($this->getParameter('exception_id')){
      $exceptionIdField= $this->getParameter('exception_id');
      $exceptionId=$this->getContext()->getRequest()->getParameter($exceptionIdField);
    }
    
    //transforming the dates to MySQL format
    list($d, $m, $y) = sfI18N::getDateForCulture($startDate, $culture);
    $startDate = strtotime("$y-$m-$d 00:00");
    list($d, $m, $y) = sfI18N::getDateForCulture($endDate, $culture);
    $endDate = strtotime("$y-$m-$d 00:00");
    
    //defining the common search criteria
    
    $c= new Criteria();
    if($this->getParameter('criteria')){
    	foreach($this->getParameter('criteria') as $criteria){
    		if(isset($criteria['field'])){
    			if(isset($criteria['criteria'])){
            eval ('$c->add('.$className.'Peer::'.$criteria['column'].', '.$this->getContext()->getRequest()->getParameter($criteria['field']).', '.$criteria['criteria'] .');');
    			}else{
            eval ('$c->add('.$className.'Peer::'.$criteria['column'].', '.$this->getContext()->getRequest()->getParameter($criteria['field']).');');
    			}
    		}else{
          if(isset($criteria['criteria'])){
            eval ('$c->add('.$className.'Peer::'.$criteria['column'].', '.$criteria['value'].', '.$criteria['criteria'] .');');
          }else{
            eval ('$c->add('.$className.'Peer::'.$criteria['column'].', '.$criteria['value'].');');
          }
    		}
    	}
    }
    if($this->getParameter('exception_id')){
      eval ('$c->add('.$className.'Peer::ID, '.$exceptionId.', Criteria::NOT_EQUAL);');
    }
    
    //first checking: start_date from the form is not in a period
    $c1= clone $c;
    eval ('$c1->add('.$className.'Peer::'.$startDateDb.', '.$startDate.', Criteria::LESS_EQUAL);');
    eval ('$c1->add('.$className.'Peer::'.$endDateDb.', '.$startDate.', Criteria::GREATER_EQUAL);');
    eval ('$foundPeriod1= '.$className.'Peer::doSelectOne($c1);');
    if($foundPeriod1){
      $error = $this->getContext()->getI18N()->__($this->getParameter('start_date_error_message')).' '.$foundPeriod1;
      return false;
    }
    
    //second checking: end_date from the form is not in a period
    $c2= clone $c;
    eval ('$c2->add('.$className.'Peer::'.$startDateDb.', '.$endDate.', Criteria::LESS_EQUAL);');
    eval ('$c2->add('.$className.'Peer::'.$endDateDb.', '.$endDate.', Criteria::GREATER_EQUAL);');
    eval ('$foundPeriod2= '.$className.'Peer::doSelectOne($c2);');
    if($foundPeriod2){
      $error = $this->getContext()->getI18N()->__($this->getParameter('end_date_error_message')).' '.$foundPeriod2;
      return false;
    }
    
    //third checking: start_date and end_date from the form do not envolve any period
    $c3= clone $c;
    eval ('$c3->add('.$className.'Peer::'.$startDateDb.', '.$startDate.', Criteria::GREATER_EQUAL);');
    eval ('$criterion3 = $c->getNewCriterion('.$className.'Peer::'.$startDateDb.', '.$endDate.', Criteria::LESS_EQUAL);'); 
    $c3->addAnd($criterion3);
    eval ('$foundPeriod3= '.$className.'Peer::doSelectOne($c3);');
    if($foundPeriod3){
      $error = $this->getContext()->getI18N()->__($this->getParameter('inside_error_message')).' '.$foundPeriod3;
      return false;
    }
    
    $c4= clone $c;
    eval ('$c4->add('.$className.'Peer::'.$endDateDb.', '.$startDate.', Criteria::GREATER_EQUAL);');
    eval ('$criterion4 = $c->getNewCriterion('.$className.'Peer::'.$endDateDb.', '.$endDate.', Criteria::LESS_EQUAL);'); 
    $c4->addAnd($criterion4);
    eval ('$foundPeriod4= '.$className.'Peer::doSelectOne($c4);');
    if($foundPeriod4){
      $error = $this->getContext()->getI18N()->__($this->getParameter('inside_error_message')).' '.$foundPeriod4;
      return false;
    }
    
    
  	return true;
  }
}
?>